import numpy as np
import xarray as xr

from struct import pack

import os
import argparse


# NOTE:
# Binary file format:
# 1 int (4-byte): number of bytes for the header
# 8 int (4-byte): start and end date, number of days and version number
# 1 int (4-byte): number of bytes for the header
# 1 int (4-byte): number of bytes for the sst array
# 360*180 float (4-byte): sst array
# 1 int (4-byte): number of bytes for the sst array
# 1 int (4-byte): number of bytes for the normalized error variance array
# 360*180 float (4-byte): normalized error variance array
# 1 int (4-byte): number of bytes for the normalized error variance array
# 1 int (4-byte): number of bytes for the ice concentration
# 360*180 char (1-byte): ice concentration
# 1 int (4-byte): number of bytes for the ice concentration


def main(fname_reynsst_in, fname_gdas_in, fname_out):

    ds = xr.open_dataset(fname_reynsst_in)
    gdas = xr.open_dataset(fname_gdas_in, engine='cfgrib',
                           backend_kwargs={'filter_by_keys': {'typeOfLevel': 'surface'},
                                           'indexpath': ''})

    gdas_surf_temp = np.flipud(gdas.t.values[:180, :]) - 273.15

    sst = ds.sst
    err = ds.err
    ice = ds.ice

    sst = sst.ffill(dim='lon', limit=2).bfill(dim='lon', limit=2)
    sst = sst.ffill(dim='lat', limit=2).bfill(dim='lat', limit=2)

    # note on header: 1:6 are start and end date. The last two are num_days and version number
    # I'm not sure if the format is correct, this is derived from the oisst.yyyymmdd file.
    # I don't think the header is used though
    date = os.path.basename(fname_reynsst_in).split('.')[1]
    header = [int(date[:4]), int(date[4:6]), int(date[6:]),
              int(date[:4]), int(date[4:6]), int(date[6:]),
              1, 1116120445]

    header = np.array(header, dtype='i4')

    rescaled_sst = np.squeeze(sst.coarsen(lat=4).mean().coarsen(lon=4).mean().values)
    rescaled_err = np.squeeze(err.coarsen(lat=4).mean().coarsen(lon=4).mean().values)
    rescaled_ice = np.squeeze(ice.coarsen(lat=4).mean().coarsen(lon=4).mean().values)

    rescaled_sst[np.isnan(rescaled_sst)] = gdas_surf_temp[np.isnan(rescaled_sst)]

    rescaled_sst = np.array(rescaled_sst, dtype='f4').ravel()
    rescaled_err = np.array(rescaled_err, dtype='f4').ravel()
    rescaled_ice = np.array(rescaled_ice, dtype='i2').ravel()

    f = open(fname_out, 'wb')
    f.write(pack('>i', 32))
    f.write(pack('>8i', *header))
    f.write(pack('>i', 32))

    f.write(pack('>i', 259200))
    f.write(pack('>64800f', *rescaled_sst))
    f.write(pack('>i', 259200))

    f.write(pack('>i', 259200))
    f.write(pack('>64800f', *rescaled_err))
    f.write(pack('>i', 259200))

    f.write(pack('>i', 64800))
    f.write(pack('>64800b', *rescaled_ice))
    f.write(pack('>i', 64800))
    f.close()


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Tool to convert the Optimum Interpolation SST ' +
                                     'provided by NCEI-NOAA at 0.25deg resolution in netcdf4 ' +
                                     'into 1deg resolution binary files compatible with the old ' +
                                     'format from NCEP that is used by the MODIS Cloud Mask.')
    parser.add_argument('sst', help='Daily Reynolds OISST file in netcdf4 format at 0.25 deg ' +
                                    'resolution.')
    parser.add_argument('gdas', help='GDAS file used to fill surface temperature over land')
    parser.add_argument('out', help='Output file name')

    args = parser.parse_args()

    main(args.sst, args.gdas, args.out)
